@@ -30,6 +30,8 @@
 // make test ARGS="-V -I 1,1"
 
 // ./tests/catch2-only-tests [Aa] -s
+// ./tests/catch2-only-tests [AaCode] -s
+// ./tests/catch2-only-tests [AaStringCodec] -s
 
 
 #include <catch2/catch.hpp>
@@ -37,12 +39,15 @@
 #include <QString>
 #include <pappsomspp/mzrange.h>
 #include <pappsomspp/amino_acid/aa.h>
+#include <pappsomspp/amino_acid/aacode.h>
+#include <pappsomspp/amino_acid/aastringcodec.h>
 #include <iostream>
 #include <pappsomspp/obo/obopsimod.h>
 #include <pappsomspp/obo/filterobopsimodtermlabel.h>
 #include <pappsomspp/obo/filterobopsimodsink.h>
 #include <pappsomspp/obo/filterobopsimodtermdiffmono.h>
 #include <pappsomspp/exception/exceptionnotfound.h>
+#include <pappsomspp/exception/exceptionoutofrange.h>
 
 using namespace pappso;
 using namespace std;
@@ -206,29 +211,29 @@ TEST_CASE("Amino Acid test suite.", "[Aa]")
       }
     /*
      * A 71.037113785565
-5:  R 156.101111025652
-5:  C 103.009184785565
-5:  D 115.026943024685
-5:  E 129.042593089167
-5:  F 147.068413914529
-5:  G 57.021463721083
-5:  H 137.058911859647
-5:  I 113.084063979011
-5:  K 128.094963016052
-5:  L 113.084063979011
-5:  M 131.040484914529
-5:  N 114.042927442166
-5:  P 97.052763850047
-5:  Q 128.058577506648
-5:  R 156.101111025652
-5:  S 87.032028405125
-5:  T 101.047678469607
-5:  V 99.068413914529
-5:  W 186.07931295157
-5:  Y 163.063328534089
-5:  U 168.964198469607
-5:  O 255.158291550141
-*/
+    5:  R 156.101111025652
+    5:  C 103.009184785565
+    5:  D 115.026943024685
+    5:  E 129.042593089167
+    5:  F 147.068413914529
+    5:  G 57.021463721083
+    5:  H 137.058911859647
+    5:  I 113.084063979011
+    5:  K 128.094963016052
+    5:  L 113.084063979011
+    5:  M 131.040484914529
+    5:  N 114.042927442166
+    5:  P 97.052763850047
+    5:  Q 128.058577506648
+    5:  R 156.101111025652
+    5:  S 87.032028405125
+    5:  T 101.047678469607
+    5:  V 99.068413914529
+    5:  W 186.07931295157
+    5:  Y 163.063328534089
+    5:  U 168.964198469607
+    5:  O 255.158291550141
+    */
     REQUIRE(pappso::Aa('A').getMass() == Approx(71.0371137856));
     REQUIRE(pappso::Aa('R').getMass() == Approx(156.101111025652));
     REQUIRE(pappso::Aa('C').getMass() == Approx(103.009184785565));
@@ -245,3 +250,131 @@ TEST_CASE("Amino Acid test suite.", "[Aa]")
   return 0;
   */
 }
+
+
+TEST_CASE("Amino Acid code test suite.", "[AaCode]")
+{
+  // Set the debugging message formatting pattern.
+  qSetMessagePattern(QString("%{file}@%{line}, %{function}(): %{message}"));
+
+  SECTION("..:: AA code init ::..", "[AaCode]")
+  {
+    AaCode aa_code;
+
+    REQUIRE(aa_code.getAa('C').getMass() ==
+            Approx(103.0091849597).margin(0.00001));
+    REQUIRE(aa_code.getAaCode('C') == 7);
+
+    REQUIRE(aa_code.getAa((uint8_t)6).getMass() <
+            aa_code.getAa((uint8_t)7).getMass());
+
+    aa_code.addAaModification('C', AaModification::getInstance("MOD:00397"));
+    
+    
+    REQUIRE(aa_code.getAaCode('M') == 13);
+    REQUIRE(aa_code.getAaCode('A') == 2);
+    REQUIRE(aa_code.getAaCode('I') == 7);
+    
+    REQUIRE(aa_code.getAa('C').getMass() ==
+            Approx(160.0306486807).margin(0.00001));
+    REQUIRE(aa_code.getAaCode('C') == 17);
+
+    REQUIRE(aa_code.getAa((uint8_t)16).getMass() <
+            aa_code.getAa((uint8_t)17).getMass());
+
+
+    REQUIRE(aa_code.getAa((uint8_t)17).getMass() <
+            aa_code.getAa((uint8_t)18).getMass());
+
+    REQUIRE(aa_code.getAa((uint8_t)1).getLetter() == 'G');
+    REQUIRE(aa_code.getAa((uint8_t)19).getLetter() == 'W');
+
+    REQUIRE_THROWS_AS(aa_code.getAa((uint8_t)20).getMass(),
+                      ExceptionOutOfRange);
+  }
+}
+
+
+TEST_CASE("Amino Acid string codec test suite.", "[AaStringCodec]")
+{
+  // Set the debugging message formatting pattern.
+  qSetMessagePattern(QString("%{file}@%{line}, %{function}(): %{message}"));
+
+  SECTION("..:: AA string codec init ::..", "[AaStringCodec]")
+  {
+    AaCode aa_code;
+    aa_code.addAaModification('C', AaModification::getInstance("MOD:00397"));
+
+    AaStringCodec codec(aa_code);
+
+
+    REQUIRE(codec.code("MAMI") == 61253);
+
+    std::uint64_t nbaa = aa_code.getSize() + 1;
+    REQUIRE(codec.code("SAM") ==
+            (aa_code.getAaCode('S') + aa_code.getAaCode('A') * (uint64_t)nbaa +
+             aa_code.getAaCode('M') * (uint64_t)nbaa * (uint64_t)nbaa));
+
+    REQUIRE(codec.codeLlc("SAM") == 873);
+    REQUIRE(codec.decode(873).toStdString() == "MSA");
+
+    REQUIRE(codec.getMass(873) ==
+            Approx(codec.getMass(5243)).margin(0.0000001));
+
+    std::vector<CodeToMass> arr_codemass =
+      codec.generateLlcCodeListByMaxPeptideSize(3);
+
+    REQUIRE(arr_codemass.size() == 1330);
+
+    std::sort(
+      arr_codemass.begin(),
+      arr_codemass.end(),
+      [](const CodeToMass &a, const CodeToMass &b) { return a.mass < b.mass; });
+    REQUIRE(codec.decode(arr_codemass.at(0).code).toStdString() == "GGG");
+    REQUIRE(codec.decode(arr_codemass.back().code).toStdString() == "WWW");
+
+
+    arr_codemass = codec.generateLlcCodeListByMaxPeptideSize(4);
+
+    REQUIRE(arr_codemass.size() == 7315);
+
+    std::sort(
+      arr_codemass.begin(),
+      arr_codemass.end(),
+      [](const CodeToMass &a, const CodeToMass &b) { return a.mass < b.mass; });
+    REQUIRE(codec.decode(arr_codemass.at(0).code).toStdString() == "GGGG");
+    REQUIRE(codec.decode(arr_codemass.back().code).toStdString() == "WWWW");
+
+
+    arr_codemass = codec.generateLlcCodeListByMaxPeptideSize(5);
+
+    REQUIRE(arr_codemass.size() == 33649);
+
+    std::sort(
+      arr_codemass.begin(),
+      arr_codemass.end(),
+      [](const CodeToMass &a, const CodeToMass &b) { return a.mass < b.mass; });
+    REQUIRE(codec.decode(arr_codemass.at(0).code).toStdString() == "GGGGG");
+    REQUIRE(codec.decode(arr_codemass.back().code).toStdString() == "WWWWW");
+
+    qDebug() << std::numeric_limits<uint32_t>::max();
+    qDebug() << std::numeric_limits<uint64_t>::max();
+    REQUIRE(codec.getLimitMax(5) < std::numeric_limits<uint32_t>::max());
+    REQUIRE(codec.getLimitMax(6) < std::numeric_limits<uint32_t>::max());
+    REQUIRE(codec.getLimitMax(7) < std::numeric_limits<uint32_t>::max());
+    REQUIRE_FALSE((uint64_t)codec.getLimitMax(8) <
+                  (uint64_t)std::numeric_limits<uint32_t>::max());
+    REQUIRE(codec.getLimitMax(5) == 3199999);
+
+
+    arr_codemass = codec.generateLlcCodeListUpToMaxPeptideSize(5);
+    std::sort(
+      arr_codemass.begin(),
+      arr_codemass.end(),
+      [](const CodeToMass &a, const CodeToMass &b) { return a.mass < b.mass; });
+    REQUIRE(codec.decode(arr_codemass.at(0).code).toStdString() == "G");
+    REQUIRE(codec.decode(arr_codemass.back().code).toStdString() == "WWWWW");
+
+    REQUIRE(arr_codemass.size() == 42503);
+  }
+}
