@@ -45,6 +45,7 @@
 #include <pappsomspp/protein/peptidefixedmodificationbuilder.h>
 #include <pappsomspp/protein/peptidevariablemodificationreplacement.h>
 #include <pappsomspp/protein/peptidemodificatorpipeline.h>
+#include <pappsomspp/protein/proteinintegercode.h>
 #include <QDebug>
 #include <QtCore>
 #include "config.h"
@@ -219,9 +220,9 @@ TEST_CASE("Test Protein", "[Protein]")
     DigestionHandler digestion;
     qDebug() << "kinase.eat(protein_sp,digestion)";
     kinase.eat(0, protein_sp, false, digestion);
-    
-    
-    REQUIRE (digestion.size() == 243);
+
+
+    REQUIRE(digestion.size() == 243);
 
     PeptideSizeFilter peptide_size(7, 35);
     peptide_size.setSink(&digestion);
@@ -229,8 +230,8 @@ TEST_CASE("Test Protein", "[Protein]")
     qDebug() << " kinase.eat(protein_sp,peptide_size);";
     digestion.clear();
     kinase.eat(0, protein_sp, false, peptide_size);
-    
-    REQUIRE (digestion.size() == 191);
+
+    REQUIRE(digestion.size() == 191);
 
     qDebug() << " modification_sink";
 
@@ -260,7 +261,7 @@ TEST_CASE("Test Protein", "[Protein]")
 
     digestion.clear();
     kinase.eat(0, protein_sp, false, peptide_size2mod);
-    REQUIRE (digestion.size() == 0);
+    REQUIRE(digestion.size() == 0);
 
     qDebug() << " kinase.eat(protein_sp,peptide_size2mod) ; end";
 
@@ -323,11 +324,10 @@ TEST_CASE("Test Protein", "[Protein]")
     qDebug() << "kinase.eat(protein_sp,semi_tryptic)";
     digestion.clear();
     kinase.eat(0, protein_sp, false, semi_tryptic);
-    
-    
-    REQUIRE(digestion.size() == 1132);
 
 
+    REQUIRE(digestion.size() == 1132);
+
 
     qDebug() << Qt::endl << "..:: Test enzyme motif ::.." << Qt::endl;
     Enzyme motif_digest("(MAMEE[KR])([^P])");
@@ -488,3 +488,56 @@ TEST_CASE("Test Protein", "[Protein]")
     REQUIRE(modification_sink.contain(peptide_test));
   }
 }
+
+
+TEST_CASE("Test Protein code", "[proteincode]")
+{
+  // Set the debugging message formatting pattern.
+  qSetMessagePattern(QString("%{file}@%{line}, %{function}(): %{message}"));
+
+  SECTION("..:: Test Protein ::..", "[proteincode]")
+  {
+
+    ProteinSp fake =
+      Protein("fakeBSA", "MVVKKVVVMAMEEKWVTFISLLLLFTKVHKECCVVSTQTALA")
+        .makeProteinSp();
+
+    AaCode aa_code;
+    aa_code.addAaModification('C', AaModification::getInstance("MOD:00397"));
+
+    AaStringCodec codec(aa_code);
+
+
+    ProteinIntegerCode protein_code(fake, codec, 5);
+
+    auto code5 = protein_code.getPeptideCodedFragment(5);
+
+    REQUIRE(code5.back() == 338550);
+
+    REQUIRE(codec.decode(code5.back()).toStdString() == "QITAA");
+
+    REQUIRE(code5.size() == fake.get()->size() - 4);
+
+    std::vector<std::uint32_t> test_code_list;
+    test_code_list.push_back(codec.codeLlc("MAMEE"));
+
+    test_code_list.push_back(codec.codeLlc("MAME"));
+
+    auto match = protein_code.match(test_code_list);
+
+    REQUIRE(match.back().first == 5);
+
+    REQUIRE(match.back().second == 8);
+
+    test_code_list.clear();
+    test_code_list.push_back(codec.codeLlc("MV"));
+
+    match = protein_code.match(test_code_list);
+
+    REQUIRE(match.front().first == 2);
+    REQUIRE(match.front().second == 0);
+    
+    REQUIRE(match.back().first == 2);
+    REQUIRE(match.back().second == 7);
+  }
+}
